import React, {useEffect, useState, useRef} from 'react';

export const SearchableSelect = ({options, onChange, value, placeholder}) => {
    const [searchTerm, setSearchTerm] = useState('');
    const [selectedValue, setSelectedValue] = useState(null);
    const [isMenuOpen, setIsMenuOpen] = useState(false);
    const inputRef = useRef(null);

    const handleButtonClick = () => {
        if (inputRef.current && !isMenuOpen) {
            inputRef.current.focus();
        }
        setIsMenuOpen(!isMenuOpen);
    };

    useEffect(() => {
        const option = options.find(option => option.value === value);
        if (option) {
            setSelectedValue(option);
        }
    }, [value, options]);

    const filteredOptions = options.filter((option) =>
        option.label.toLowerCase().includes(searchTerm.toLowerCase())
    );

    const handleSelect = (option) => {
        setSelectedValue(option);
        setIsMenuOpen(false);
        onChange(option);
        setSearchTerm('');
    };

    const clearSelection = () => {
        setSelectedValue(null);
        setSearchTerm('');
        onChange(null);
    };

    return (
        <div className="relative">
            <div
                className="flex items-center w-56 border rounded-md shadow-sm focus-within:ring-2 focus-within:ring-blue-500">
                <input
                    ref={inputRef}
                    type="text"
                    value={isMenuOpen ? searchTerm : (selectedValue ? selectedValue.label : '')}
                    onChange={(e) => setSearchTerm(e.target.value)}
                    placeholder={isMenuOpen ? (selectedValue ? selectedValue.label : '') : placeholder}
                    onFocus={() => setIsMenuOpen(true)}
                    onBlur={() => setTimeout(() => {
                        setSearchTerm('');
                        setIsMenuOpen(false)
                    }, 300)}
                    className={`w-full h-10 px-5 py-2 border border-r-white border-r-0 rounded-br-none rounded-tr-none rounded-md shadow-sm focus:outline-none  sm:text-sm ${isMenuOpen ? 'placeholder-black' : ''}`}
                />

                {selectedValue && (
                    <button
                        onClick={clearSelection}
                        type="button"
                        className="py-2 h-10 border-t border-b focus:outline-none bg-white"
                        aria-label="Clear selection"
                    >
                        <svg
                            xmlns="http://www.w3.org/2000/svg"
                            className="w-5 h-5 text-gray-400 hover:text-gray-600"
                            fill="none"
                            viewBox="0 0 24 24"
                            stroke="currentColor"
                        >
                            <path
                                strokeLinecap="round"
                                strokeLinejoin="round"
                                strokeWidth={2}
                                d="M6 18L18 6M6 6l12 12"
                            />
                        </svg>
                    </button>
                )}

                <button
                    onClick={handleButtonClick}
                    type="button"
                    className="p-2 h-10 border-l-0 border focus:outline-none bg-white rounded-br-md rounded-tr-md"
                >
                    <svg
                        className={`w-5 h-5 text-gray-400 hover:text-gray-600 transition-transform ${
                            isMenuOpen ? 'rotate-180' : ''
                        }`}
                        xmlns="http://www.w3.org/2000/svg"
                        viewBox="0 0 20 20"
                        fill="currentColor"
                    >
                        <path
                            fillRule="evenodd"
                            d="M5.293 7.707a1 1 0 011.414 0L10 11.586l3.293-3.879a1 1 0 111.414 1.414l-4 4a1 1 0 01-1.414 0l-4-4a1 1 0 010-1.414z"
                            clipRule="evenodd"
                        />
                    </svg>
                </button>
            </div>

            {isMenuOpen && (
                <div
                    className="absolute z-10 w-full mt-2 bg-white border rounded-md shadow-lg max-h-60 overflow-y-auto sm:text-sm">
                    {filteredOptions.length > 0 ? (
                        filteredOptions.map((option) => (
                            <div
                                key={option.value}
                                onClick={() => handleSelect(option)}
                                className={selectedValue && selectedValue.value === option.value ? 'px-4 py-2 cursor-pointer bg-blue-500 text-white' : 'px-4 py-2 cursor-pointer hover:bg-blue-200 hover:text-black'}
                            >
                                {option.label}
                            </div>
                        ))
                    ) : (
                        <div className="px-4 py-2 text-gray-500">No results found</div>
                    )}
                </div>
            )}
        </div>
    );
};
